{% import 'keg_elements/forms/horizontal.html' as dynamic_render %}
{% import 'keg_elements/forms/horizontal_static.html' as static_render %}

<div id="dynamic">
    <div id="with-caller">
        {% call dynamic_render.section('My Heading', form) %}
            {{ dynamic_render.field(form[field_name]) }}
        {% endcall %}
    </div>
    <div id="with-field-name">
        {{ dynamic_render.section('My Heading', form, [field_name]) }}
    </div>
</div>

<div id="static">
    <div id="with-caller">
        {% call static_render.section('My Heading', form) %}
            {{ static_render.field(form[field_name]) }}
        {% endcall %}
    </div>
    <div id="with-field-name">
        {{ static_render.section('My Heading', form, [field_name]) }}
    </div>
</div>
